//***************************************************************************************
// Function definitions for Breakout
// Stephen Anderson
// March 02, 2004
//***************************************************************************************

#include "proto.h"
int fps = 0;									//Initialize frames per second counter

//***************************************************************************************
// Get the raw joystick values and adjust create a deadzone in the center (SDK CODE)
//***************************************************************************************
void adjust_for_joystick_deadzone( int x_deadzone, int *joy_x )
{
register int x;

	x = *joy_x;

	if( x > 0  && x < x_deadzone )
		x = 0;
	else if( x > 0 )
		x -= x_deadzone;

	if( x < 0 && x > (-x_deadzone) )
		x = 0;
	else if( x < 0 )
		x += x_deadzone;

	*joy_x = x;
}

//***************************************************************************************
// Set a few variables and set the ball(s) in motion
//***************************************************************************************
void ball_serve (int a){						//Serve ball[a]
												//xdif used to make ball speed (X axis) progressive
	if (level >= 1 && level <=3) xdif = 0;
	if (level >= 4 && level <=9) xdif = 1;
	if (level >= 10) xdif = 2;
	
	theball[a].xpos = (rand() % 320) + 30;		//Random x position (between 30 and 320)
	theball[a].ypos = (105 - (rand() % 65));	//Random y position (between 105 and 40)
	theball[a].dx = ((rand() % 3) + 1) + xdif;	//Random theball[a].dx
	if( rand() & 0x0001 )						//theball[a].dx is either left or right
		theball[a].dx = -theball[a].dx;
	theball[a].dy = 2;
	theball[a].active = 1;						//Put ball in motion
	theball[a].collision = 0;					//Ball cannot yet hit bricks

	if (a == 0)									//Following code prevents multiple launches
		num_hits = 0;							//of ball[X]
	else										//if the ball is lost before a paddle hit
		num_hits++;								//occurs.

	num_balls_active++;
}

//***************************************************************************************
// Remove a brick from screen, handle collision detection stuff (2600 Breakout style)
//***************************************************************************************
void remove_brick (int brk, int value, int a){
	SPRRemoveSprite (bricks[brk].BrickSpr);
	bricks[brk].visible = 0;
	theball[a].collision = 0;	//Allows collision detection to be turned off after a brick is hit
	numbricks--;
	Pan.PCMPanLR = LEFT + ((theball[a].xpos - 23) * PAN);	//Pan L-R based on ball position (X-axis)
	switch (value){
	case 1:
		PlayWAV((void *)BrikWav, -1, &Pan, VOLUME, ECHO );
		score += (1 * num_balls_active);
		break;
	case 2:
		PlayWAV((void *)Brk3Wav, -1, &Pan, VOLUME, ECHO );
		score += (3 * num_balls_active);
		break;
	case 3:
		PlayWAV((void *)Brk5Wav, -1, &Pan, VOLUME, ECHO );
		score += (5 * num_balls_active);
		break;
	}//Switch
}

//***************************************************************************************
// Should be self explanatory
//***************************************************************************************
void ball_miss (int a){							//Ball[a] has missed paddle								
	int sound;									//Phrase randomizer

	Pan.PCMPanLR = LEFT + ((theball[a].xpos - 23) * PAN);	//Pan L-R based on ball position (X-axis)

	theball[a].xpos = 0;						//Hide ball
	theball[a].ypos = 0;
	theball[a].active = 0;						//Stop ball motion

	if (a == 0){
		num_hits = 0;							//Reset ball speed counter(Y axis)
		lives--;								//and lose a life only on losing ball[0]
	}											//else, just continue on like nothing happened
	
	//Game over code is in main game loop - trying to keep things running fast

	sound = (rand() % 4);						//Choose a random wav file

	switch (sound){
	case 0:
		PlayWAV((void *)HahaWav, -1, &Pan, VOLUME, ECHO );
		break;
	case 1:
		PlayWAV((void *)Doh1Wav, -1, &Pan, VOLUME, ECHO );
		break;
	case 2:
		PlayWAV((void *)Doh2Wav, -1, &Pan, VOLUME, ECHO );
		break;
	case 3:
		PlayWAV((void *)DamnWav, -1, &Pan, VOLUME, ECHO );
		break;
	}//SWITCH

	num_balls_active--;
}

//***************************************************************************************
// Should be self explanatory
//***************************************************************************************
int level_start (void){
	int i, j, num = 0;			//Counter variables
	for (i=0; i< 3; i++){		//ROWS
		for (j=0; j<15; j++){	//COLS
			SPRAddSprite (bricks[num].BrickSpr, ((j*24) + 34), ((i*18) + 50), 1, 0x4000, 0x4000, kSpriteSimple, kBlack, 1);
			bricks[num].x = ((j*24) + 34);
			bricks[num].y = ((i*18) + 50);
			bricks[num].visible = 1;
			num++;
		}//Inner Loop
	}//Outer Loop
	
	for (i=0; i < MAX_BALLS; i++){	//BAD IDEA TO RE-USE VARIABLES!!!!!
		theball[i].active = 0;
		theball[i].collision = 0;}
	level++;

	num_balls_active = 0;
	num_hits = 0;

	return num;					//Returns number of bricks
}

//***************************************************************************************
// Checks for ball to paddle collisions and creates rebound angle
//***************************************************************************************
void check_paddle (int a) {
	int delta;

	if (theball[a].active){							//Only check active balls
		if (theball[a].ypos > 218){
			delta = (theball[a].xpos - thepaddle.xpos);
			if ( abs (delta) <= (diff == 0 ? 38 : 25) ){				//Ball has hit paddle
				//LEFT EDGE OF PADDLE
				if (delta <=(diff == 0 ? -23 : -15) && delta >=(diff == 0 ? -38 : -25) ){	//Sharp angle at paddle edge
					theball[a].dx = ((-3) - xdif);
				}
				if (delta <=(diff == 0 ? -11 : -7) && delta >=(diff == 0 ? -14 : -22)){		//Medium angle at paddle inside
					theball[a].dx = ((-2) - xdif);
				}
				if (delta <=-1 && delta >=(diff == 0 ? -10 : -6) ){		//Small angle at paddle just-off-center(TM)
					theball[a].dx = ((-1) - xdif);
				}
				//MIDDLE OF PADDLE
				if (delta == 0){					//Reflect at paddle center
					theball[a].dx = -theball[a].dx;}
				//RIGHT EDGE OF PADDLE
				if (delta <=(diff == 0 ? 38 : 25) && delta >=(diff == 0 ? 23 : 15) ){		//Sharp angle at paddle edge
					theball[a].dx = (3 + xdif);
				}
				if (delta <=(diff == 0 ? 22 : 14) && delta >=(diff == 0 ? 11 : 7)){		//Medium angle at paddle inside
					theball[a].dx = (2 + xdif);
				}
				if (delta <=(diff == 0 ? 10 : 6) && delta >=1){		//Small angle at paddle just-off-center(TM)
					theball[a].dx = (1 + xdif);
				}
				theball[a].dy = -(theball[a].dy);
				theball[a].collision = 1;			//Allow ball to hit bricks again
				num_hits++;
				Pan.PCMPanLR = LEFT + ((theball[a].xpos - 23) * PAN);	//Pan L-R based on ball position (X-axis)
				PlayWAV((void *)PadlWav, -1, &Pan, VOLUME, ECHO );
			}//If test - paddle hit
			else	//Ball has missed paddle
				ball_miss(a);
		}//If test - range test for collision detect
	}//If test - ball active
}

//***************************************************************************************
// Check for ball to brick collision.  I am sure this can be tweaked!
// I tried optimizing the code.  I got it down to 1 loop and 40 lines.  The OBJ file
// was larger and the game ran slower - go figure.  See FOO.TXT for the alternate code
//***************************************************************************************
void check_bricks (void) {
	int counter, a;

	//Only check brick collision when theball.ypos between 90 & 20
	//Separate checks for ball going up and ball coming down will save computation also (I think)

	//1st - CHECK WHEN BALL GOING UP (bottom of brick collisions)
	for (a=0; a < MAX_BALLS; a++) {
		if (theball[a].active) {
			if (theball[a].dy < 0){
				if (theball[a].ypos <= 98 && theball[a].ypos >=  86 && theball[a].collision == 1){	//Check bottom row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter + 30].visible == 1){	//Brick hit
							remove_brick (counter + 30, 1, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - bottom row check
				}//If test for bottom row

				if (theball[a].ypos <= 80 && theball[a].ypos >=  68 &&  theball[a].collision == 1){	//Check middle row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter + 15].visible == 1 ){	//Brick hit
							remove_brick (counter + 15, 2, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - middle row check
				}//If test - middle row check
		
				if (theball[a].ypos <= 62 && theball[a].ypos >=  50 &&  theball[a].collision == 1){	//Check top row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter].visible == 1){	//Brick hit
							remove_brick (counter, 3, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - top row check
				}//If test - top row check
			}//If test - ball going UP

			//Now, CHECK WHEN BALL GOING DOWN (top of brick theball[a].collisions)
			if (theball[a].dy > 0){	
				if (theball[a].ypos >= 36 && theball[a].ypos <=  52 && theball[a].collision == 1){	//Check top row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter].visible == 1 ){	//Brick hit
							remove_brick (counter, 3, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - top row check
				}//If test - top row check

				if (theball[a].ypos >= 54 && theball[a].ypos <=  70 &&  theball[a].collision == 1){	//Check middle row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter + 15].visible == 1 ){	//Brick hit
							remove_brick (counter + 15, 2, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - middle row check
				}//If test - middle row check

				if (theball[a].ypos >= 72 && theball[a].ypos <=  88 &&  theball[a].collision == 1){	//Check bottom row
					for (counter = 0; counter < 15; counter++){
						if (theball[a].xpos >= ((counter + 1)  *  24) && theball[a].xpos <= (((counter + 1) * 24) + 20) && bricks[counter + 30].visible == 1 ){	//Brick hit
							remove_brick (counter + 30, 1, a);
							theball[a].dy = -(theball[a].dy);
							return;
						}
					}//For loop - bottom row check
				}//If test - bottom row check
			}//If test - ball going down
		}//If test - active ball check inside loop
	}//For loop - multiple ball loo
}

//***************************************************************************************
// Moves ball, handles multi-ball, handles ball speed.
//***************************************************************************************
void move_ball (void){
	int a;

	for (a=0; a < MAX_BALLS; a++) {
		if (theball[a].active) {
			//Edge detection for rebound with no other action (Left, Right, Top of screen)
			if (theball[a].xpos > 369 || theball[a].xpos < 24){
				theball[a].dx = -(theball[a].dx);					//Detect screen edge and rebound ball
				Pan.PCMPanLR = LEFT + ((theball[a].xpos - 23) * PAN);	//Pan L-R based on ball position (X-axis)
				PlayWAV((void *)WallWav, -1, &Pan, VOLUME, ECHO );
			}
			if (theball[a].ypos < 33){
				theball[a].dy = -(theball[a].dy);					//Detect top screen edge and rebound ball
				theball[a].collision = 1;								//Allow hectic play after "breakthrough"
				Pan.PCMPanLR = LEFT + ((theball[a].xpos - 23) * PAN);	//Pan L-R based on ball position (X-axis)
				PlayWAV((void *)WallWav, -1, &Pan, VOLUME, ECHO );
			}

			theball[a].xpos += theball[a].dx;
			theball[a].ypos += theball[a].dy;

			check_paddle(a);									//Check for paddle theball[a].collision
	
			check_bricks();									//Check for brick theball[a].collision

			SPRSetSpriteXY (theball[a].BallSpr, theball[a].xpos, theball[a].ypos);

			//Code for progressive speed & multi-ball
			if (num_hits == (diff == 0 ? 3 : 5) ){							//Easy - 3, hard - 5
				if (theball[0].dy > 0) theball[0].dy = 3;
				if (theball[0].dy < 0) theball[0].dy = -3;
			}
			if (num_hits == (diff == 0 ? 8 : 10) ){							//Easy - 8, hard - 10
				if (theball[0].dy > 0) theball[0].dy = 4;
				if (theball[0].dy < 0) theball[0].dy = -4;
				if (theball[1].active == 0){
					ball_serve(1);
					theball[1].dy = 3;	//Ball always starts at top & falls down
				}//if - multiball code
			}
			if (num_hits == (diff == 0 ? 11 : 15) ){						//Easy - 11, hard - 15
				if (theball[0].dy > 0) theball[0].dy = 5;
				if (theball[0].dy < 0) theball[0].dy = -5;
				if (theball[1].active == 0){
					ball_serve(1);
					theball[1].dy = 3;	//Ball always starts at top & falls down
				}//if - multiball code
				if (theball[2].active == 0){
					ball_serve(2);
					theball[2].dy = 4;	//Ball always starts at top & falls down				
				}//if - multiball code
			}
		}//If test - active ball
	}//FOR
}
	
//***************************************************************************************
// Allows user to change analog deadzone and maximum paddle speed
// Displays a really bad starfield (hey - it was my 1st use of fixed point)
//***************************************************************************************
void ConfigControls(int *X_Speed, int *DeadZone)
{
	static int inputdelay = 0;
	register int xs, xd, i, a;
	int num_stars;										//Number of stars being rendered (NOTE - Array of Stars must be filled from 0 to (num_stars - 1)

	xs = *X_Speed;
	xd = *DeadZone;
	
	//Setup the starfield
	xorg = SCREENWIDTH / 2;								//Center star onscreen
	yorg = SCREENHEIGHT / 2;							//Center star onscreen

	//Setup the starfield
	for (a=0; a < MAX_STARS; a++){
		stars[a].x = (rand() % SCREENWIDTH)  << 4;
		stars[a].y = (rand() % SCREENHEIGHT) << 4;
		stars[a].dx = (rand() % 15);					//Negative velocity is allowed
		if (stars[a].dx == 0)							//Zero velocity is not
			stars[a].dx = 1;
		if( stars[a].x < (SCREENWIDTH / 2)  << 4 )		//Necessary because center of starfield is NOT at (0,0)
			stars[a].dx = -stars[a].dx;					//Stars left of center will move left
		stars[a].dy = (rand() % 15);					//Negative velocity is allowed
		if (stars[a].dy == 0)							//Zero velocity is not
			stars[a].dy = 1;
		if( stars[a].y < (SCREENHEIGHT / 2) << 4 )		//Necessary because center of starfield is NOT at (0,0)
			stars[a].dy = -stars[a].dy;					//Stars above center will move up
		stars[a].color = clut[rand() % 26];				//Pick random values from a pre-generated color table
	}

	num_stars = 275;

	//Remove any "leftover" bricks after the game has finished
	for (i=0; i < MAX_BRICKS;i++)
	{
		if (bricks[i].visible)
		{
			SPRRemoveSprite (bricks[i].BrickSpr);
			bricks[i].visible = 0;
		}//if
	}//for

	while (1){
		if( inputdelay )
		{
			inputdelay--;	// Debounce period
		}
		else
		{
			if( ButtonCDown(_Controller[1]) )
			{
				xs++;
				if( xs > 15 )
					xs = 15;
	
				inputdelay = INPUTDELAY_PERIOD;
			}//CDown
			else if (ButtonCUp(_Controller[1]) )
			{
				xs--;
				if( xs < 1 )
					xs = 1;
		
				inputdelay = INPUTDELAY_PERIOD;
			}//CUp
			else if (ButtonCRight(_Controller[1]) )
			{
				xd+=4;
				if( xd > 48 )
					xd = 48;
        
				inputdelay = INPUTDELAY_PERIOD;
			}//CRight
			else if (ButtonCLeft(_Controller[1]) )
			{
				xd-=4;
				if( xd < 12 )
					xd = 12;
        
				inputdelay = INPUTDELAY_PERIOD;
			}//CLeft
			else if (ButtonStart(_Controller[1]) )
				return;								//Return to menu, which will return to main (at startgame label)
		}//else

		*X_Speed = xs;
		*DeadZone = xd;

		theScreen = &gl_screenbuffers[drawBuffer];
		ClearScreen(theScreen);

		//Blit the starfield
		for (a=0;a < num_stars;a++){
			stars[a].x += stars[a].dx;
			stars[a].y += stars[a].dy;
			if (stars[a].x >= (SCREENWIDTH << 4) || stars[a].x <= (0 << 4) || stars[a].y >= (SCREENHEIGHT << 4) || stars[a].y <= (0 << 4) ){
				stars[a].x = xorg << 4;		//Reset star to origin
				stars[a].y = yorg  << 4;	//Reset star to origin
			}//if
			//Now that we have range checked, blit the stars to the screen
			_DMABiLinear( theScreen->dmaFlags|DMA_DIRECT_BIT, theScreen->memP, ( 1 <<16)| (stars[a].x) >> 4,
				( 1 <<16)| (stars[a].y >> 4), (void *)stars[a].color);
		}//for

//	This code drops me to 46 FPS - WHY WHY WHY PLEASE TELL ME WHY

		//Line 1
		msprintf( buf, "Controller Config" );
		m2dSetRect( &r, 100, 15, 360, 40 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, AtariScriptP, 14, clr_blue, clr_black, eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		//Line 2 - part 1
		msprintf( buf, "delta X:" );
		m2dSetRect( &r, 25, 55, 70, 65 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, sysP, 10, clr_red, clr_black, eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );
		
		//Line 2 - part 2
		msprintf( buf, "%d", deltax );
		m2dSetRect( &r, 70, 55, 85, 65 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		//Line 3 - Part 1
		msprintf( buf, "X speed:" );
		m2dSetRect( &r, 25, 65, 70, 75 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		//Line 3 - Part 2
		msprintf( buf, "%d", 16-xs );
		m2dSetRect( &r, 70, 65, 85, 75 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );
		
		//Line 4 - Part 1
		msprintf( buf, "X deadzone:" );
		m2dSetRect( &r, 25, 75, 70, 85 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		//Line 4 - Part 2
		msprintf( buf, "%d", xd );
		m2dSetRect( &r, 70, 75, 85, 85 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		//Keep these down here because it saves several calls to mmlSetTextProperties
		mmlSetTextProperties( fc, sysP, 10, clr_light_blue, clr_black, eOpaque, 0, 0 );

		//Line 3 - Part 3
		msprintf( buf, "C--Up / Down   to adjust." );
		m2dSetRect( &r, 85, 65, 200, 75 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );
	
		//Line 4 - Part 3
		msprintf( buf, "C--Left / Right to adjust." );
		m2dSetRect( &r, 85, 75, 200, 85 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );
	
		//Line 5
		msprintf( buf, "Press start to begin game" );
		m2dSetRect( &r, 160, 210, 260, 220 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], buf, strlen(buf), &r );

		MovePaddle();

		SPRSetDestScreen(gl_screenbuffers[drawBuffer].memP, 
						 gl_screenbuffers[drawBuffer].dmaFlags,  
						 0,  0,  SCREENWIDTH-1,  SCREENHEIGHT-1, 
						 kBlack);
		SPRDraw (0,1);

		VideoSync();
	}//while(1)
}

//***************************************************************************************
// Displays the game options menu
//***************************************************************************************
void menu(void) {

	static int inputdelay = 0;					//Debounce the controller
	int selected = 0;							//Line of text to be "highlighted"
	int txt_size = 12;
	textCode status[50];

	SPR_IMAGE_INFO			Menu_logo;
	
	SPRInstallTGAImage (MenuScreen, 1, &Menu_logo, 0x00);
	if (DEBUG){
		theScreen = &gl_screenbuffers[0];
		ClearScreen(theScreen);
		mmlSimpleVideoSetup(&gl_screenbuffers[0], &gl_sysRes, eTwoTapVideoFilter);	//Make sure message gets displayed
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 115, 0x51F05B00, "Menu_logo TGA");
		msprintf( status, "Width = %02d, Height = %02d", Menu_logo.width, Menu_logo.height );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 130, 0x51F05B00, status);
		msprintf( status, "Mem pos %d", Menu_logo.img );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 145, 0x51F05B00, status);
		SPRBlitter(thepaddle.Paddle.img, thepaddle.Paddle.dmaflags, 0, 0, 80, 60,
               gl_screenbuffers[0].memP, gl_screenbuffers[0].dmaFlags,
			   SCREENWIDTH/2, 200, 0x10000, 0x10000, 0,
               0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
               0x40000002, 0);
		_TimeToSleep (5000);	//Pause 15 seconds to display info
	}// If (DEBUG)

	if (DEBUG){
		theScreen = &gl_screenbuffers[0];
		ClearScreen(theScreen);
		mmlSimpleVideoSetup(&gl_screenbuffers[0], &gl_sysRes, eTwoTapVideoFilter);	//Make sure message gets displayed
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 115, 0x51F05B00, "Paddle brokenTGA");
		msprintf( status, "Width = %02d, Height = %02d", thepaddle.Paddle.width, thepaddle.Paddle.height );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 130, 0x51F05B00, status);
		msprintf( status, "Mem pos %d", thepaddle.Paddle.img );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 145, 0x51F05B00, status);
		SPRBlitter(thepaddle.Paddle.img, thepaddle.Paddle.dmaflags, 0, 0, 80, 60,
               gl_screenbuffers[0].memP, gl_screenbuffers[0].dmaFlags,
			   SCREENWIDTH/2, 200, 0x10000, 0x10000, 0,
               0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
               0x40000002, 0);
		_TimeToSleep (10000);	//Pause 15 seconds to display info
	}// If (DEBUG)
	
	while ( 1 ){
		theScreen = &gl_screenbuffers[drawBuffer];
		ClearScreen(theScreen);
		
		SPRBlitter(Menu_logo.img, Menu_logo.dmaflags, 0, 0, 300, 40,
               gl_screenbuffers[drawBuffer].memP, gl_screenbuffers[drawBuffer].dmaFlags,
			   SCREENWIDTH/2, 35, 0x9000, 0x9000, 0,
               0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
               0x40000002, 0);
		
		/*
		SPRBlitter(Title.img, Title.dmaflags, 0, 0, 250, 31,
               gl_screenbuffers[drawBuffer].memP, gl_screenbuffers[drawBuffer].dmaFlags,
			   (SCREENWIDTH/2 - 0), 35, 0x10000, 0x10000, 0,
               0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
               0x40000002, 0);
		*/
		if( inputdelay )
		{
			inputdelay--;	// Debounce period
		}
		else
		{
			if( ButtonUp(_Controller[1]) )
			{
				selected--;
				if( selected < 0 )		//Let it "wrap around"
					selected = 2;
	
				inputdelay = INPUTDELAY_PERIOD;
			}//Up
			else if (ButtonDown(_Controller[1]) )
			{
				selected++;
				if( selected > 2 )		//Let it "wrap around"
					selected = 0;
		
				inputdelay = INPUTDELAY_PERIOD;
			}//Down
			else if (ButtonA(_Controller[1]) )
			{
				switch (selected){
				case 0:
						diff = 1 - diff;	//Toggle between 1 and 0 with each key press (Thanks Skah T for the tip!)
						//Scale paddle according to difficulty
						SPRSetSpriteScale (thepaddle.PaddleSpr, (diff == 0 ? 0xC000 : 0x8000), 0x8000);
						inputdelay = INPUTDELAY_PERIOD;
						break;
				case 1: 
						ConfigControls(&xspeed, &xdeadzone);	//Diff has already been set at this point
						return;									//Program control returns to "startgame" label in main					    
				case 2: return;									//Program control returns to "startgame" label in main
				}//SWITCH
			}//else if
		}//else

		//Print text to the screen
		//Set up 1st half of line 1 and print it
		msprintf( status, "Set Difficulty" );
		m2dSetRect( &r, 30, 75, 360, 100 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_light_blue, bakColor = (selected !=0 ? clr_black : clr_blue), eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );
		
		//Set up 2nd half of line 1 and print it
		if (diff) msprintf( status, "HARD" );
		else msprintf ( status, "EASY" );
		m2dSetRect( &r, 250, 75, 360, 100 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_light_red, clr_black, eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

		//Set up line 2 and print it
		msprintf( status, "Configure Controller" );
		m2dSetRect( &r, 30, 110, 360, 135 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_light_blue, bakColor = (selected !=1 ? clr_black : clr_blue), eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

		//Set up line 3 and print it
		msprintf( status, "Start Game" );
		m2dSetRect( &r, 30, 145, 360, 170 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_light_blue, bakColor = (selected !=2 ? clr_black : clr_blue), eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

		//Set up line 4 and print it
//		msprintf( status, "Credits" );
//		m2dSetRect( &r, 30, 180, 360, 205 );	//MAKE AS SMALL AS POSSIBLE !!!
//		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_light_blue, bakColor = (selected !=3 ? clr_black : clr_blue), eOpaque, 0, 0 );
//		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

		SPRWait();		//Make sure all blitting is done before we flip the screen buffer
		VideoSync();	//Flip the buffer
	}//while(1)

	_MemFree (Menu_logo.img);		//Free up the memory used by the menu logo
}

//***************************************************************************************
// Displays the game title screen for 10 seconds
//***************************************************************************************
void title(void) {

	SPR_IMAGE_INFO	Title;			// Holds info on the title screen TGA
	textCode		status[50];		// Just a text buffer

	SPRInstallTGAImage (TitleScreen, 1, &Title, 0x00);
	if (DEBUG){
		theScreen = &gl_screenbuffers[0];
		ClearScreen(theScreen);
		mmlSimpleVideoSetup(&gl_screenbuffers[0], &gl_sysRes, eTwoTapVideoFilter);	//Make sure message gets displayed
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 115, 0x51F05B00, "Title TGA");
		msprintf( status, "Width = %02d, Height = %02d", Title.width, Title.height );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 130, 0x51F05B00, status);
		msprintf( status, "Mem pos %d", Title.img );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 145, 0x51F05B00, status);
		SPRBlitter(thepaddle.Paddle.img, thepaddle.Paddle.dmaflags, 0, 0, 80, 60,
               gl_screenbuffers[0].memP, gl_screenbuffers[0].dmaFlags,
			   SCREENWIDTH/2, 200, 0x10000, 0x10000, 0,
               0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
               0x40000002, 0);
		_TimeToSleep (5000);	//Pause 15 seconds to display info
	}// If (DEBUG)

	// We "hardcode" the buffer indices below because we are displaying
	// to a static buffer.  This way, the screen will show up!
	theScreen = &gl_screenbuffers[0];
	ClearScreen(theScreen);
	SPRBlitter(Title.img, Title.dmaflags, 0, 0, 256, 160,
           gl_screenbuffers[0].memP, gl_screenbuffers[drawBuffer].dmaFlags,
		   SCREENWIDTH/2, SCREENHEIGHT/2, 0x13000, 0x13000, 0,
           0,0,SCREENWIDTH-1,SCREENHEIGHT-1,
           0x40000002, 0);

	if (DEBUG) {
		//Print build date in lower right corner of screen
		msprintf( status, "DEBUG BUILD" );
		m2dSetRect( &r, 30, 215, 360, 230 );	//MAKE AS SMALL AS POSSIBLE !!!
		mmlSetTextProperties( fc, sysP, 11, clr_red, clr_black, eOpaque, 0, 0 );
		mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );
	}

	//Print build date in lower right corner of screen
	msprintf( status, "Build date : 03/16/2004 8:48pm" );
	m2dSetRect( &r, 240, 215, 360, 230 );	//MAKE AS SMALL AS POSSIBLE !!!
	mmlSetTextProperties( fc, sysP, 11, clr_light_grey, clr_black, eOpaque, 0, 0 );
	mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

	_TimeToSleep (5000);			//Hold title screen up for 5 seconds

	_MemFree (Title.img);			//Free the memory allocated for the title screen

	menu();
}

//***************************************************************************************
// Displays the game credits
//***************************************************************************************
void Credits(void) {

	int v_pos = 240;		// Vertical position counter for scrolling text
	int v_offset;			// Vertical scroll offset
	int txt_size = 9;		// Size of text in credits
	int txt_pos[3];			// Makes code easier to read
	int h_pos1 = 30;		// Left text border
	int h_pos2 = 180;		// Offset for 2nd half of text since mml2D text output capabilities are shit
	textCode status[128];	// Character buffer
	SPR_IMAGE_INFO			Logo;
	
	SPRInstallTGAImage (LogoScreen, 1, &Logo, 0x00);
	if (DEBUG){
		theScreen = &gl_screenbuffers[0];
		ClearScreen(theScreen);
		mmlSimpleVideoSetup(&gl_screenbuffers[0], &gl_sysRes, eTwoTapVideoFilter);	//Make sure message gets displayed
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 115, 0x51F05B00, "Logo TGA");
		msprintf( status, "Width = %02d, Height = %02d", Logo.width, Logo.height );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 130, 0x51F05B00, status);
		msprintf( status, "Mem pos %d", Logo.img );
		DebugWS(theScreen->dmaFlags, theScreen->memP, 45, 145, 0x51F05B00, status);
		_TimeToSleep (5000);	//Pause 15 seconds to display info
	}// If (DEBUG)

	Pan.PCMPanLR = 0;	//Pan center
	PlayWAV((void *)PityWav, -1, &Pan, VOLUME, ECHO );

	for (v_offset = 0; v_offset <= 200; v_offset++){	//text gone at 144 - we go to 200 for delay
		theScreen = &gl_screenbuffers[drawBuffer];
		ClearScreen(theScreen);

		SPRBlitter(Logo.img, Logo.dmaflags, 0, 0, 256, 160,
				gl_screenbuffers[drawBuffer].memP, gl_screenbuffers[drawBuffer].dmaFlags,
				SCREENWIDTH/2, SCREENHEIGHT/2, 0x14000, 0x14000, 0,
				0,0,SCREENWIDTH-1,SCREENHEIGHT-1, 0x40000002, 0);

		//Only need to do this once for the 4 lines of credits
		mmlSetTextProperties( fc, AtariScriptP, txt_size, clr_yellow, clr_black, eBlend, 0, 0 );
		
		//Set up text position arrays for printing and range checking
		txt_pos[0] = (v_pos - v_offset );
		txt_pos[1] = (v_pos + (2 * txt_size) - v_offset );
		txt_pos[2] = (v_pos + (4 * txt_size) - v_offset );
		txt_pos[3] = (v_pos + (6 * txt_size) - v_offset );

		//Print text to the screen if it is within range
		//Set up line 1 and print it if within range
		if ( (txt_pos[0] - v_offset) > 20 && (txt_pos[0] - v_offset) < 240 ) {
			msprintf( status, "Game code" );
			m2dSetRect( &r, h_pos1, (txt_pos[0] - v_offset), 150, (txt_pos[0] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

			msprintf( status, "Stephen Anderson" );
			m2dSetRect( &q, h_pos2, (txt_pos[0] - v_offset), 360, (txt_pos[0] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &q );
		}
		
		//Set up line 2 and print it if within range
		if ( (txt_pos[1] - v_offset) > 20 && (txt_pos[1] - v_offset) < 240 ) {
			msprintf( status, "Title Screen & logo" );
			m2dSetRect( &r, h_pos1, (txt_pos[1] - v_offset) , 180, (txt_pos[1] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

			msprintf( status, "Kevin Manne" );
			m2dSetRect( &q, h_pos2, (txt_pos[1] - v_offset) , 360, (txt_pos[1] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &q );
		}

		//Set up line 3 and print it if within range
		if ( (txt_pos[2] - v_offset) > 20 && (txt_pos[2] - v_offset) < 240 ) {
			msprintf( status, "Sounds" );
			m2dSetRect( &r, h_pos1, (txt_pos[2] - v_offset) , 150, (txt_pos[2] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

			msprintf( status, "Ripped from the web" );
			m2dSetRect( &q, h_pos2, (txt_pos[2] - v_offset) , 360, (txt_pos[2] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &q );
		}

		//Set up line 4 and print it if within range
		if ( (txt_pos[3] - v_offset) > 20 && (txt_pos[3] - v_offset) < 240 ) {
			msprintf( status, "Support and testing" );
			m2dSetRect( &r, h_pos1, (txt_pos[3] - v_offset), 170, (txt_pos[3] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &r );

			msprintf( status, "Skah_T, Riff, Kevin Manne" );
			m2dSetRect( &q, h_pos2, (txt_pos[3] - v_offset), 360, (txt_pos[3] + txt_size - v_offset) );
			mmlSimpleDrawText( fc, &gl_screenbuffers[drawBuffer], status, strlen(status), &q );
		}


		if (DEBUG){
			msprintf (status, "txt_pos[0] = %d", txt_pos[0]);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 50, TEXTCOLOR, status);
			msprintf (status, "txt_pos[1] = %d", txt_pos[1]);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 65, TEXTCOLOR, status);
			msprintf (status, "txt_pos[2] = %d", txt_pos[2]);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 80, TEXTCOLOR, status);
			msprintf (status, "txt_pos[3] = %d", txt_pos[3]);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 95, TEXTCOLOR, status);
			msprintf (status, "v_offset = %d", v_offset);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 110, TEXTCOLOR, status);
			msprintf (status, "fps = %d", fps);
			DebugWS(theScreen->dmaFlags, theScreen->memP, 230, 125, TEXTCOLOR, status);
		}

		_TimeToSleep (25);		//25ms wait for a 40Hz FUR

		SPRWait();		//Make sure all blitting is done before we flip the screen buffer
		VideoSync();	//Flip the buffer
	}//for
	_TimeToSleep (3000);		//Pause for 3 seconds to laugh at Mr. T

	//This is not necessary since we reboot the system after the credits, but it's a nice thing to do.
	_MemFree (Logo.img);		//Free the memory allocated for Mr. T
}


//***************************************************************************************
// Move the paddle.  I have this in a seperate function because the game uses it
// as well as the controller configuration screen.
//***************************************************************************************
void MovePaddle(void){
	static int delay = 30;	//speed up paddle every .5 seconds (assume 60FPS game)

	//BELOW IS CODE FOR ANALOG CONTROL
	// update position based on analog joystick values 
	// these are signed 8 bit values 
	deltax = JoyXAxis(_Controller[1]);
			
	adjust_for_joystick_deadzone( xdeadzone, &deltax );

	//BELOW IS THE CODE FOR D-PAD SUPPORT.
	//We use the deltax variable to avoid problems from hitting analog & d-pad at same time
	//If D-pad is not moved, we use the analog value
	//Else we use D-pad value
	//Acceleration doesn't work - each frame, deltax gets reset.  Think of a way to fix this.
	
	//ACCELERATION FIX 01/26/2004 - JESUS THIS WAS SIMPLE.  WHY DIDN'T I SEE IT BEFORE?!?
	//WHILE BUTTON IS BEING HELD DOWN, INCREMENT A TIMER VALUE (BY .01, OR SOME SIMILAR SMALL VALUE)
	//ADD THIS SMALL VALUE TO THE DELTA VARIABLE.  RESET THE VALUE UPON LETTING UP BUTTON
	//DON'T LET DELTA VALUE GO ABOVE A MAXIMUM VALUE (+-60)
	//WE WILL ONLY IMPLEMENT ACCELERATION TO THE D-PAD!!!
	if (ButtonLeft (_Controller[1])){
		deltax = -60;
		if (delay){
			delay--;	//PAUSE
		}
		else{
			deltax--;	//accelerate paddle left
			delay = 30;	//reset counter
		}
	}
	if (ButtonRight (_Controller[1])){
		deltax = 60;
		if (delay){
			delay--;	//PAUSE
		}
		else{
			deltax++;	//accelerate paddle right
			delay = 30;	//reset counter
		}
	}
			
	// Scale back the joystick values to create a delta, then update position
	//thepaddle.xpos += deltax / (diff == 0 ? 5 : 4);			//Easy - 5, Hard - 4 (smaller # = faster)
	thepaddle.xpos += deltax / xspeed;		//user configured - change with difficulty?? - MAYBE

	// Make sure sprite is still on screen
	// Stop when hit boundary
	if (thepaddle.xpos < 45)
		thepaddle.xpos = 45;
	else if (thepaddle.xpos > 360)
		thepaddle.xpos = 360;

	// update sprite
	SPRSetSpriteXY(thepaddle.PaddleSpr, thepaddle.xpos, 223);
}

//***************************************************************************************
// A nice collection of graphics utilities needed for just about any application
// All of this stuff was taken from the SDK
//***************************************************************************************

void ClearScreen(mmlDisplayPixmap *scrn) // Erase the screen by drawing 8x8 rectangle blocks; 
{										 // Not perfectly efficient, but pretty good
long x, y;

    for (x = 0; x < scrn->wide; x += 8)
	{
		for (y = 0; y < scrn->high; y += 8)
		{
			_DMABiLinear(scrn->dmaFlags|DMA_DIRECT_BIT, scrn->memP, (8<<16)|x, (8<<16)|y, (void *)BACKGROUNDCOLOR);
		}
    }
}

void init_screenbuffers() //Initialize the draw/display buffers, clear the memory, put one up!
{
	mmlInitDisplayPixmaps( &gl_screenbuffers[0], &gl_sysRes, SCREENWIDTH, SCREENHEIGHT, e888Alpha, 1, NULL );
	mmlInitDisplayPixmaps( &gl_screenbuffers[1], &gl_sysRes, SCREENWIDTH, SCREENHEIGHT, e888Alpha, 1, NULL );
	mmlInitDisplayPixmaps( &gl_screenbuffers[2], &gl_sysRes, SCREENWIDTH, SCREENHEIGHT, e888Alpha, 1, NULL );
	
	ClearScreen(&gl_screenbuffers[0]);	//Generally a bad idea to hard code index
	ClearScreen(&gl_screenbuffers[1]);	//Generally a bad idea to hard code index
	ClearScreen(&gl_screenbuffers[2]);	//Generally a bad idea to hard code index

	// Point the video hardware at the display buffer
	mmlSimpleVideoSetup(&gl_screenbuffers[drawBuffer], &gl_sysRes, eTwoTapVideoFilter);
}

//I took this nice triple buffer routine from Chomp - by Andreas Binner
int VideoSync(void)
{
    static int frames;
    static int lastfield;
    static int lastsecond;
    int ret;

    int curfield;

    /* we want to view the buffer we just finished
       drawing */

    /* set the video to point at the drawn buffer */
    mmlSimpleVideoSetup(&gl_screenbuffers[drawBuffer], &gl_sysRes, eTwoTapVideoFilter);

    /* find out how many fields have been drawn */
    curfield = _VidSync(-1);

    /* remember that we've drawn a frame */
    frames++;

    /* have more than 60 fields (1 second) passed? */
    if (curfield - lastsecond >= 60) {
        fps = frames;
        frames = 0;
        lastsecond = curfield;
    }

    /* draw to a new buffer */
    drawBuffer++;
    if (drawBuffer >= NUM_BUFFERS)
        drawBuffer = 0;

#if NUM_BUFFERS == 2
    /* wait until a new field starts (so it's no longer
       showing the field we're going to draw on) */
    curfield = _VidSync(0);
#endif

    ret = curfield - lastfield;
    while (ret == 0) {
        curfield = _VidSync(-1);
        ret = curfield - lastfield;
    }
    lastfield = curfield;
    return ret;
}