//***************************************************************************************
// Function prototypes for Breakout
// Stephen Anderson
// March 02, 2004
//***************************************************************************************

#ifndef _PROTO_H_
#define _PROTO_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <nuon/mml2d.h>
#include <nuon/nise.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdarg.h>
#include <nuon/dma.h>
#include <nuon/mutil.h>
#include <nuon/msprintf.h>
#include <nuon/bios.h>
#include <nuon/mml2d.h>
#include <nuon/sprite.h>
#include <nuon/nise.h>
#include "colors.h"

//Default Values

#define SCREENWIDTH			(392)
#define SCREENHEIGHT 		(240)

#define INPUTDELAY_PERIOD	(25)		//Larger # = slower repeat on the controller

#define LEFT_MARGIN			(40)

#define BACKGROUNDCOLOR 	kBlack
#define TEXTCOLOR 			0x7276c600	//Light Blue (64,64,255) RGB

#define MAX_BRICKS			(45)
#define MAX_BALLS			(3)

#define MAX_STARS			(350)		//2000 works but is somewhat crowded :)

#define NUM_BUFFERS			(3)

#define LEFT				0xC0000000	//PCM full left
#define FRONT				0x40000000	//PCM full front
#define PAN					0x5D9F73	//Max range / #ball positions (350) - used for L-R panning
#define VOLUME				0x15000000	//Max is 0x40000000 - we are at 33%
#define ECHO				0x0			//Max is 0x40000000

#define DEBUG				0			//1 = display debugging text, 0 = don't display debugging text

//2/14/2002
//Brick structure - holds sprite image and (x,y) position
typedef struct
{
	SPRITE					*BrickSpr;
	int						x, y, visible;
} BRICK;

typedef struct
{
	SPRITE					*BallSpr;
	SPR_IMAGE_INFO			Ball;
	int						xpos, ypos, dx, dy, active, collision;
} BALL;

typedef struct
{
	SPRITE					*PaddleSpr;
	SPR_IMAGE_INFO			Paddle;
	int						xpos;
} PADDLE;

//10/13/2002
typedef struct
{
	int			x, y;		//Position
	int			dx, dy;		//Velocity
	mmlColor	color;		//Color (We'll use a pre-generated table)
} STAR;

// globals
mmlGC					gl_gc;
mmlSysResources 		gl_sysRes;
mmlDisplayPixmap		*theScreen;
mmlFontContext			fc;
mmlFont					sysP;
mmlFont					AtariScriptP;
mmlTextStyle			bk;
mmlColor				bakColor;
m2dRect					r,q;
extern int				drawBuffer;
mmlDisplayPixmap		gl_screenbuffers[NUM_BUFFERS];
extern int				fps;
int						deltax, xpos, ypos;
extern int				numbricks;
int						score, level, num_hits, xdif, lives;
extern int				num_balls_active;		//Obvious and also used as a score multiplier
extern int				diff;					//Game difficulty
int						xspeed, xdeadzone;		//used in controller config
extern int				change;					//Used in menu rendering code (title screen)
SPR_IMAGE_INFO			BrickMatrix;
BRICK					bricks[MAX_BRICKS];
BALL					theball[MAX_BALLS];
PADDLE					thepaddle;
char					buf[SPRINTF_MAX];
PCMPOS					Pan;
void *					pad;					//Needed for a _MemAlloc call
//Starfield Stuff - global because it is used both inside and outside of the main loop
STAR stars[MAX_STARS];
extern mmlColor			clut[27];
int						xorg, yorg;				//Starfield origin coordinates (0<=xorg<=319 && 0<=yorg<=239)

extern uint8 SysFont[];
extern uint8 AtariScript[];

//  Binaries (see "data.s")
//  Sprite Image Data
extern char BallSprite[];
extern char PaddleSprite[];
extern char TitleScreen[];
extern char LogoScreen[];
extern char MenuScreen[];
extern char BricksSprite[];		//9X1 matrix of bricks each 64x48 pixels
//  Sounds
extern char HahaWav[];
extern char Doh1Wav[];
extern char Doh2Wav[];
extern char DamnWav[];
extern char BrikWav[];
extern char Brk3Wav[];
extern char Brk5Wav[];
extern char WallWav[];
extern char PadlWav[];
extern char PityWav[];

//***************************************************************************************
// Audio Stuff
// PlayWAV taken from Open SDK
//***************************************************************************************
int PlayWAV(void *wavaddr, int voice, PCMPOS *Pan, int volume, int echo );

//***************************************************************************************
// Graphics Stuff (SEE "graphics.c")
//***************************************************************************************
void ClearScreen(mmlDisplayPixmap *scrn);

void init_screenbuffers();

//I took this nice triple buffer routine from Chomp - by Andreas Binner
int VideoSync(void);

//***************************************************************************************
// Gameplay Stuff (SEE "gamestuff.c")
//***************************************************************************************
void adjust_for_joystick_deadzone( int x_deadzone, int *joy_x );

void MovePaddle (void);

void ball_serve (int a);

void remove_brick (int brk, int value, int a);

void ball_miss (int a);

int level_start (void);

void check_paddle (int a);

void check_bricks (void);

void move_ball (void);

void ConfigControls (int *X_Speed, int *DeadZone);

void menu (void);

void title (void);

void MovePaddle(void);

void Credits (void);

int main (void);

#ifdef __cplusplus
}
#endif

#endif